"""Absorbers module — inject extracted knowledge into target models."""

from model_osmosis.absorbers.base import BaseAbsorber
from model_osmosis.absorbers.pytorch_absorber import PyTorchAbsorber
from model_osmosis.absorbers.sklearn_absorber import SklearnAbsorber

__all__ = [
    "BaseAbsorber",
    "PyTorchAbsorber",
    "SklearnAbsorber",
]
