"""Extractors module — pull knowledge out of trained models."""

from model_osmosis.extractors.base import BaseExtractor
from model_osmosis.extractors.pytorch_extractor import PyTorchExtractor
from model_osmosis.extractors.sklearn_extractor import SklearnExtractor
from model_osmosis.extractors.generic_extractor import GenericExtractor

__all__ = [
    "BaseExtractor",
    "PyTorchExtractor",
    "SklearnExtractor",
    "GenericExtractor",
]
