"""Core module — configuration, exceptions, and base classes."""

from model_osmosis.core.config import OsmosisConfig, GrowthStrategyType
from model_osmosis.core.exceptions import (
    OsmosisError,
    ExtractionError,
    AbsorptionError,
    GrowthError,
    IncompatibleModelError,
)

__all__ = [
    "OsmosisConfig",
    "GrowthStrategyType",
    "OsmosisError",
    "ExtractionError",
    "AbsorptionError",
    "GrowthError",
    "IncompatibleModelError",
]
