"""Stacking module — Mixture-of-Experts model stacking with optimization."""

from model_osmosis.stacking.stacked_model import StackedModel
from model_osmosis.stacking.stacker import ModelStacker
from model_osmosis.stacking.optimizer import StackOptimizer
from model_osmosis.stacking.offloader import ExpertOffloader
from model_osmosis.stacking.safe_stacker import SafeStacker

__all__ = [
    "StackedModel",
    "ModelStacker",
    "StackOptimizer",
    "ExpertOffloader",
    "SafeStacker",
]
